from behave import step_registry
from behave.runner import ModelRunner, Context
from django.shortcuts import resolve_url


class PatchedContext(Context):

    @property
    def base_url(self):
        assert hasattr(self.test, 'live_server_url'), (
            'Web browser automation is not available.'
            ' This scenario step can not be run'
            ' with the --simple or -S flag.')
        return self.test.live_server_url

    def get_url(self, to=None, *args, **kwargs):
        return self.base_url + (
            resolve_url(to, *args, **kwargs) if to else '')


class BehaveHooksMixin(object):
    """
    Provides methods that run during test execution

    These methods are attached to behave via monkey patching.
    """
    testcase_class = None

    def patch_context(self, context):
        """
        Patches the context to add utility functions

        Sets up the base_url, and the get_url() utility function.
        """
        context.__class__ = PatchedContext
        # Simply setting __class__ directly doesn't work
        # because behave.runner.Context.__setattr__ is implemented wrongly.
        object.__setattr__(context, '__class__', PatchedContext)

    def before_scenario(self, context):
        """
        Method that runs immediately before behave's before_scenario function

        Sets up the test case.
        """
        context.test = self.testcase_class()

        if getattr(context, 'fixtures', None):
            context.test.fixtures = context.fixtures

        if getattr(context, 'reset_sequences', None):
            context.test.reset_sequences = context.reset_sequences

        if hasattr(context, 'scenario'):
            self.load_registered_fixtures(context=context)

        context.test._pre_setup(run=True)
        context.test.setUpClass()
        context.test()

    def after_scenario(self, context):
        """
        Method that runs immediately after behave's after_scenario function
        """
        context.test.tearDownClass()
        context.test._post_teardown(run=True)
        del context.test

    def load_registered_fixtures(self, context):
        """
        Apply fixtures that are registered with the @fixtures decorator.
        """
        for step in context.scenario.all_steps:
            match = step_registry.registry.find_match(step)
            if match and hasattr(match.func, 'registered_fixtures'):
                if not context.test.fixtures:
                    context.test.fixtures = []
                context.test.fixtures.extend(match.func.registered_fixtures)


def monkey_patch_behave(django_test_runner):
    """
    Integrate behave_django in behave via before/after scenario hooks
    """
    behave_run_hook = ModelRunner.run_hook

    def run_hook(self, name, context, *args):
        if name == 'before_all':
            django_test_runner.patch_context(context)

        behave_run_hook(self, name, context, *args)

        if name == 'before_scenario':
            django_test_runner.before_scenario(context)

        if name == 'after_scenario':
            django_test_runner.after_scenario(context)

    ModelRunner.run_hook = run_hook



from os import chdir
from os.path import abspath, dirname, join, normpath
from setuptools import find_packages, setup


def read_file(filename):
    with open(join(dirname(abspath(__file__)), filename)) as f:
        return f.read()


# allow setup.py to be run from any path
chdir(normpath(abspath(dirname(__file__))))

import behave_django  # noqa

setup(
    name='behave-django',
    version=behave_django.__version__,
    packages=find_packages(exclude=['test*']),
    include_package_data=True,
    license=behave_django.__license__,
    description=behave_django.__doc__,
    long_description=read_file('README.rst'),
    url='https://github.com/behave/behave-django',
    author='Mitchel Cabuloy',
    author_email='mixxorz@gmail.com',
    maintainer='Mitchel Cabuloy, Peter Bittner',
    maintainer_email='mixxorz@gmail.com',
    install_requires=read_file('requirements.txt'),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Environment :: Plugins',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Testing',
    ],
    test_suite='tests',
)
